package org.foobar;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.UUID;

import javax.inject.Inject;
import javax.sql.DataSource;

import org.apache.commons.dbutils.DbUtils;
import org.apache.log4j.Logger;
import org.infinispan.Cache;
import org.infinispan.manager.EmbeddedCacheManager;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

@Component
public class TestDataController {
	
	@Inject
	private DataSource dataSource_;
	
	@Inject
	private TestDataDAO testDataDAO_;
	
	@Inject
	private EmbeddedCacheManager cacheManager_;

	private Logger getLogger() {
		return Logger.getLogger(getClass());
	}
	
	@Transactional
	public int getTestDataCount() throws SQLException {
		Connection connection = null;
		try {
			connection = dataSource_.getConnection();
			Statement statement = connection.createStatement();
			
			ResultSet resultSet = statement.executeQuery("select count(*) from test_data");
			resultSet.next();
			Integer result = Integer.valueOf(resultSet.getInt(1));
			
			DbUtils.closeQuietly(resultSet);
			DbUtils.closeQuietly(statement);
			
			return result;
		} finally {
			DbUtils.closeQuietly(connection);
		}
	}
	
	/**
	 * Generates 50k {@link TestData} entities
	 */
	public void buildTestData() throws SQLException {
		if (getTestDataCount() == 0) {
			for (int i = 0; i < 50; i++) {
				getLogger().info(String.format("Adding TestData %d - %d", i*1000, i*1000 + 1000));
				addTestDataBatch();
			}
		}
	}
	
	/**
	 * Add 1000 {@link TestData} entities per transaction
	 */
	@Transactional
	protected void addTestDataBatch() {
		for (int i = 0; i < 1000; i++) {
			TestData testData = new TestData();
			testData.setValue(UUID.randomUUID().toString());
			
			testDataDAO_.persist(testData);
		}
	}

	@Transactional
	public void fillCache() {
		List<TestData> testDataList = testDataDAO_.findAll();
		
		int count = 0;
		for (TestData testData : testDataList) {
			Cache<Long, String> cache = cacheManager_.getCache();
			
			if (! cache.containsKey(testData.getTestDataId())) {
				cache.put(testData.getTestDataId(), testData.getValue());
			}
			
			count++;
			
			if (count % 1000 == 0 ) {
				getLogger().info(String.format("%d test data added to cache", count));
			}
		}
	}

	public void fillCacheInBatches() {
		List<TestData> testDataList = testDataDAO_.findAll();
		
		int count = 0;
		Collection<TestData> testDataCollection = new ArrayList<TestData>();
		for (TestData testData : testDataList) {
			testDataCollection.add(testData);
			
			count++;
			
			if (count % 50000 == 0 ) {
				runCacheBatch(testDataCollection);
				
				testDataCollection = new ArrayList<TestData>();
				getLogger().info(String.format("%d test data batch added to cache", count));
			}
		}
	}
	
	@Transactional
	public void runCacheBatch(Collection<TestData> testDataCollection) {
		for (TestData testData : testDataCollection) {
			Cache<Long, String> cache = cacheManager_.getCache();
			
			if (! cache.containsKey(testData.getTestDataId())) {
				cache.put(testData.getTestDataId(), testData.getValue());
			}
		}
	}
}
