package org.foobar;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.UUID;

import javax.inject.Inject;

import org.apache.log4j.Logger;
import org.infinispan.Cache;
import org.infinispan.manager.EmbeddedCacheManager;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;
import org.springframework.test.context.transaction.TransactionConfiguration;
import org.springframework.transaction.annotation.Transactional;

@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(locations = { "classpath*:beans.xml" })
@TransactionConfiguration(transactionManager = "transactionManager", defaultRollback = false)
public class TXCacheTest {

	@Inject
	private TestDataDAO testDataDAO_;
	
	@Inject
	private TestDataController testDataController_;

	@Inject
	private EmbeddedCacheManager cacheManager_;

	private Logger logger_ = Logger.getLogger(getClass());

	/**
	 * - create a bunch (50k rows) of test data - read each row of test data - for each row, look to see if test data is in the cache - if not in the cache, add the test data
	 */
	@Test
	public void testWithEntities() throws Exception {
		logger_.info("starting testWithEntities");
		long start = System.currentTimeMillis();
		
		testDataController_.buildTestData();

		testDataController_.fillCache();

		logger_.info(String.format("testWithEntities complete - %dms", (System.currentTimeMillis() - start)));
	}

	@Test
	public void testWithManyBatches() throws Exception {
		logger_.info("starting testWithManyBatches");
		long start = System.currentTimeMillis();
		
		testDataController_.buildTestData();

		List<TestData> testDataList = testDataDAO_.findAll();
		
		int count = 0;
		Collection<TestData> testDataBatch = new ArrayList<TestData>();
		for (TestData testData : testDataList) {
			testDataBatch.add(testData);
			
			count++;
			
			if (count % 1000 == 0) {
				testDataController_.runCacheBatch(testDataBatch);
				
				testDataBatch.clear();
				logger_.info(String.format("%d test data batch added to cache", count));
			}
		}
		

		logger_.info(String.format("testWithManyBatches complete - %dms", (System.currentTimeMillis() - start)));
	}

	@Test
	public void testWithOneBatch() throws Exception {
		logger_.info("starting testWithOneBatch");
		long start = System.currentTimeMillis();
		
		testDataController_.buildTestData();

		List<TestData> testDataList = testDataDAO_.findAll();
		
		testDataController_.runCacheBatch(testDataList);

		logger_.info(String.format("testWithOneBatch complete - %dms", (System.currentTimeMillis() - start)));
	}

	@Test
	@Transactional
	public void testNoEntities() throws Exception {
		logger_.info("starting testNoEntities");

		long start = System.currentTimeMillis();

		int count = 0;
		Cache<Long, String> cache = cacheManager_.getCache("testNoEntities");
		for (long l = 0L; l < 50000L; l++) {
			Long key = Long.valueOf(l);

			if (!cache.containsKey(key)) {
				cache.put(key, UUID.randomUUID().toString());
			} else {
				cache.get(key);
			}

			count++;

			if (count % 1000 == 0) {
				logger_.info(String.format("%d test data added to cache", count));
			}
		}

		logger_.info(String.format("testNoEntities complete - %dms", (System.currentTimeMillis() - start)));
	}
}
